"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.das = exports.dasPluginsToCorePlugins = void 0;
const umi_1 = require("@metaplex-foundation/umi");
const generated_1 = require("./generated");
const MPL_CORE_ASSET = 'MplCoreAsset';
const MPL_CORE_COLLECTION = 'MplCoreCollection';
function convertSnakeCase(str, toCase = 'camel') {
    return str
        .toLowerCase()
        .split('_')
        .map((word, index) => toCase === 'camel' && index === 0
        ? word
        : word.charAt(0).toUpperCase() + word.slice(1))
        .join('');
}
function getUpdateAuthority(groupingItem, authority) {
    const result = {
        updateAuthority: { type: 'None' },
    };
    if (groupingItem && groupingItem.group_key === 'collection') {
        result.updateAuthority = {
            type: 'Collection',
            address: (0, umi_1.publicKey)(groupingItem.group_value),
        };
    }
    else {
        result.updateAuthority = {
            type: 'Address',
            address: authority.address,
        };
    }
    return result;
}
function getAccountHeader(executable, lamps, rentEpoch) {
    return {
        header: {
            executable: executable ?? false,
            owner: generated_1.MPL_CORE_PROGRAM_ID,
            lamports: (0, umi_1.lamports)(lamps ?? -1),
            ...(rentEpoch !== undefined ? { rentEpoch } : {}),
        },
    };
}
function getRuleSet(dasRuleSet) {
    const isRuleSetString = typeof dasRuleSet === 'string';
    const ruleSetKind = isRuleSetString ? dasRuleSet : Object.keys(dasRuleSet)[0];
    const ruleSetData = !isRuleSetString
        ? dasRuleSet[ruleSetKind].map((bytes) => (0, umi_1.publicKey)(new Uint8Array(bytes)))
        : [];
    if (ruleSetKind === 'program_allow_list') {
        return (0, generated_1.ruleSet)('ProgramAllowList', [ruleSetData]);
    }
    if (ruleSetKind === 'program_deny_list') {
        return (0, generated_1.ruleSet)('ProgramDenyList', [ruleSetData]);
    }
    return (0, generated_1.ruleSet)('None');
}
function dasPluginDataToCorePluginData(dasPluginData) {
    return (({ basis_points, creators, rule_set, attribute_list, frozen, additional_delegates, }) => ({
        ...(basis_points !== undefined ? { basisPoints: basis_points } : {}),
        ...(creators !== undefined
            ? {
                creators: creators.map((creator) => ({
                    ...creator,
                    address: (0, umi_1.publicKey)(creator.address),
                })),
            }
            : {}),
        ...(rule_set !== undefined ? { ruleSet: getRuleSet(rule_set) } : {}),
        ...(attribute_list !== undefined ? { attributeList: attribute_list } : {}),
        ...(frozen !== undefined ? { frozen } : {}),
        ...(additional_delegates !== undefined
            ? { additionalDelegates: additional_delegates }
            : {}),
    }))(dasPluginData);
}
function dasPluginsToCorePlugins(dasPlugins) {
    return Object.keys(dasPlugins).reduce((acc, dasPluginKey) => {
        const dasPlugin = dasPlugins[dasPluginKey];
        const { authority, data, offset } = dasPlugin;
        const authorityAddress = authority?.address;
        acc = {
            ...acc,
            [convertSnakeCase(dasPluginKey)]: {
                authority: {
                    type: authority.type,
                    ...(authorityAddress ? { address: (0, umi_1.publicKey)(authorityAddress) } : {}),
                },
                ...dasPluginDataToCorePluginData(data),
                offset: BigInt(offset),
            },
        };
        return acc;
    }, {});
}
exports.dasPluginsToCorePlugins = dasPluginsToCorePlugins;
function dasAssetToCoreAssetOrCollection(dasAsset) {
    // TODO: Define types in Umi DAS client.
    const { interface: assetInterface, id, ownership: { owner }, content: { metadata: { name }, json_uri: uri, }, compression: { seq }, grouping, authorities, plugins, executable, lamports: lamps, rent_epoch: rentEpoch, mpl_core_info: mplCoreInfo, } = dasAsset;
    const { num_minted: numMinted = 0, current_size: currentSize = 0 } = mplCoreInfo ?? {};
    const commonFields = {
        publicKey: id,
        uri,
        name,
        ...getAccountHeader(executable, lamps, rentEpoch),
        ...dasPluginsToCorePlugins(plugins),
        // pluginHeader: // TODO: Reconstruct
    };
    const isCollection = assetInterface === MPL_CORE_COLLECTION;
    if (isCollection) {
        return {
            ...commonFields,
            key: generated_1.Key.CollectionV1,
            updateAuthority: authorities[0].address,
            numMinted,
            currentSize,
        };
    }
    return {
        ...commonFields,
        key: generated_1.Key.AssetV1,
        owner,
        seq: seq ? (0, umi_1.some)(BigInt(seq)) : (0, umi_1.none)(),
        ...getUpdateAuthority(grouping[0], authorities[0]),
    };
}
async function searchAssets(context, input) {
    const dasAssets = await context.rpc.searchAssets({
        ...input,
        interface: (input.interface ?? MPL_CORE_ASSET),
        burnt: false,
    });
    return dasAssets.items.map((dasAsset) => dasAssetToCoreAssetOrCollection(dasAsset));
}
function searchCollections(context, input) {
    return searchAssets(context, { ...input, interface: MPL_CORE_COLLECTION });
}
function fetchAssetsByOwner(context, input) {
    return searchAssets(context, {
        owner: input.owner,
    });
}
function fetchAssetsByAuthority(context, input) {
    return searchAssets(context, {
        ...input,
        authority: input.authority,
    });
}
function fetchAssetsByCollection(context, input) {
    return searchAssets(context, {
        ...input,
        grouping: ['collection', input.collection],
    });
}
function fetchCollectionsByUpdateAuthority(context, input) {
    return searchCollections(context, {
        ...input,
        authority: input.updateAuthority,
    });
}
exports.das = {
    searchAssets,
    searchCollections,
    fetchAssetsByOwner,
    fetchAssetsByAuthority,
    fetchAssetsByCollection,
    fetchCollectionsByUpdateAuthority,
};
//# sourceMappingURL=das.js.map